@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 # Date::HolidayParser
-# $Id: HolidayParser.pm 333 2006-06-19 10:54:42Z zero_dogg $
+# $Id: HolidayParser.pm 481 2006-08-01 16:05:49Z zero_dogg $
 # A parser of ~/.holiday-style files.
 #  The format is based off of the holiday files found bundled
 #  with the plan program, not any official spec. This because no
@@ -24,12 +24,11 @@ my @EXPORT_OK = qw(EasterCalc ParseHoliday);
 
 # Version number
 my $VERSION;
-$VERSION = 0.2;
+$VERSION = 0.3;
 
 # The user should be able to tell us to be silent
 our $BeSilent;
 
-
 # Easter calculation using the gauss algorithm. See:
 # http://en.wikipedia.org/wiki/Computus#Gauss.27s_algorithm
 # Parts of the code stolen from Kenny Fowler
@@ -38,7 +37,7 @@ our $BeSilent;
 # Usage: $EasterYDay = EasterCalc(YEAR);
 # 
 # YEAR should be absolute and not relative to 1900
-sub EasterCalc ($) {
+sub EasterCalc {
 	my $year = $_[0];
 
 	my $c;
@@ -78,7 +77,7 @@ sub EasterCalc ($) {
 
 # Purpose: Calculate a NumericYDay
 # Usage: $CreativeParser{FinalYDay} = _HCalc_NumericYDay($CreativeParser{NumericYDay}, $CreativeParser{AddDays}, $CreativeParser{SubtDays});
-sub _HCalc_NumericYDay ($$$) {
+sub _HCalc_NumericYDay {
 	my ($DAY, $ADD_DAYS, $SUBTRACT_DAYS) = @_;
 	if(defined($ADD_DAYS)) {
 		$DAY += $ADD_DAYS;
@@ -94,7 +93,7 @@ sub _HCalc_NumericYDay ($$$) {
 
 # Purpose: Return the English day name of the year day supplied
 # Usage: $DayName = _Holiday_DayName(INTEGER_NUMBER, YEAR);
-sub _Holiday_DayName ($$) {
+sub _Holiday_DayName {
 	my $year = $_[1];
 	$year -= 1900;
 	
@@ -116,7 +115,7 @@ sub _Holiday_DayName ($$) {
 
 # Purpose: Return the yday of the supplied unix time
 # Usage: $YDay = _Get_YDay(UNIX_TIME);
-sub _Get_YDay ($) {
+sub _Get_YDay {
 	my $Unix_Time = $_[0];
 	warn("_Get_YDay: Invalid usage: must be numeric. Got \"$Unix_Time\"") and return(undef) if $Unix_Time =~ /\D/;
 	my ($get_sec,$get_min,$get_hour,$get_mday,$get_mon,$get_year,$get_wday,$get_yday,$get_isdst) = localtime($Unix_Time);
@@ -125,226 +124,260 @@ sub _Get_YDay ($) {
 
 # Purpose: Print a warning about some error during the holiday parsing
 # Usage: _HolidayError(LINE_NO, FILENAME, ERROR, ACTION_TAKEN);
-sub _HolidayError ($$$$) {
+sub _HolidayError {
 	_PrintError("*** Holiday parser error: $_[2] on line $_[0] in $_[1]. $_[3]\n");
 }
 
 # Purpose: Print a syntax error in a holiday file
 # Usage: _SyntaxError(LINE_NO, FILENAME, ERROR, ACTION_TAKEN);
-sub _SyntaxError ($$$$) {
+sub _SyntaxError {
 	_PrintError("*** Holiday parser: Syntax error: $_[2] on line $_[0] in $_[1]. $_[3]\n");
 }
 
 # Purpose. Actually print the error (obeying $BeSilent)
 # Usage: _PrintError(ERROR);
-sub _PrintError($) {
+sub _PrintError {
 	unless($BeSilent) {
 		warn $_[0];
 	}
 }
 
-# Purpose: Interperate and calculate the holiday file
-# Usage: _Holiday_Interperate(\%CreativeParser, Line_Number, HolidayName, YEAR, \%FinalParsing, Filename);
-sub _Holiday_Interperate ($$$$$$) {
-	my $CreativeParser = $_[0];
-	my $LineNo = $_[1];
-	my $HolidayName = $_[2];
-	my $Year = $_[3];
-	my $FinalParsing = $_[4];
-	my $File = $_[5];
-	my %MonthMapping = (
-		'january' => 0,
-		'february' => 1,
-		'march' => 2,
-		'april' => 3,
-		'may' => 4,
-		'june' => 5,
-		'july' => 6,
-		'august' => 7,
-		'september' => 8,
-		'october' => 9,
-		'november' => 10,
-		'december' => 11,
-	);	# Hash mapping the month name to a numeric
-	
-	if(defined($CreativeParser->{MustBeDay})) {
-
-		# If IsMonth is defined then find a NumericYDay that we can use so that
-		# the NumericYDay parsing below can do all of the heavy lifting
-		if (defined($CreativeParser->{IsMonth})) {
-			my $PosixYear = $Year - 1900;
-			my $PosixTime = POSIX::mktime(0, 0, 0, 1, $MonthMapping{$CreativeParser->{IsMonth}}, $PosixYear);
-			my $proper_yday = _Get_YDay($PosixTime);
-			unless(defined($CreativeParser->{Number})) {
-				_HolidayError($LineNo, $File, "\$CreativeParser->{Number} is undef", "Skipping this line. This is probably a bug in the parser");
-				return(0);
-			}
-			if($CreativeParser->{Number} eq 'last') {
-				# Find the first of the set day
-				while(1) {
-					if(_Holiday_DayName($proper_yday, $Year) eq $CreativeParser->{MustBeDay}) {
-						last;
-					}
-					$proper_yday++;
-				}
-
-				# Find the last of the set day
-				my $Last_YDay = $proper_yday;
-				while(1) {
-					if(_Holiday_DayName($proper_yday, $Year) eq $CreativeParser->{MustBeDay}) {
-						$proper_yday += 7;
-					}
-					my $MKTime = POSIX::mktime(0, 0, 0, $proper_yday, 0, $PosixYear);
-					my ($detect_sec,$detect_min,$detect_hour,$detect_mday,$detect_mon,$detect_year,$detect_wday,$detect_yday,$detect_isdst) = localtime($MKTime);
-					# If $detect_mon is not equal to $MonthMapping{$CreativeParser->{IsMonth}} then
-					# we're now on the next month and have found the last of the day
-					unless($detect_mon eq $MonthMapping{$CreativeParser->{IsMonth}}) {
-						last;
-					}
-					$Last_YDay = $proper_yday;
-				}
-				$CreativeParser->{NumericYDay} = $Last_YDay;
-				$CreativeParser->{BeforeOrAfter} = 'before';
-			} else {
-				# Parse the final
-				$CreativeParser->{NumericYDay} = $proper_yday;
-				if($CreativeParser->{Number} eq 'first') {
-					$CreativeParser->{BeforeOrAfter} = 'after';
-				} elsif($CreativeParser->{Number} eq 'second') {
-					$CreativeParser->{BeforeOrAfter} = 'after';
-					$CreativeParser->{AddDays} = 7;
-				} elsif($CreativeParser->{Number} eq 'third') {
-					$CreativeParser->{BeforeOrAfter} = 'after';
-					$CreativeParser->{AddDays} = 14;
-				} elsif($CreativeParser->{Number} eq 'fourth') {
-					$CreativeParser->{BeforeOrAfter} = 'after';
-					$CreativeParser->{AddDays} = 21;
-				} else {
-					_HolidayError($LineNo, $File, "\$CreativeParer->{Number} is \"$CreativeParser->{Number}\"", "This is a bug in the parser. This line will be ignored") and return(undef) unless $CreativeParser->{Number} eq 'null';
-				}
-			}
-
-		}
-
-
-
-		if(defined($CreativeParser->{NumericYDay})) {
-			# Parse the main NumericYDay
-			$CreativeParser->{FinalYDay} = _HCalc_NumericYDay($CreativeParser->{NumericYDay}, $CreativeParser->{AddDays}, $CreativeParser->{SubtDays});
-			unless(defined($CreativeParser->{BeforeOrAfter})) {
-				_SyntaxError($LineNo, $File, "It was not defined if the day should be before or after", "Defaulting to before. This is likely to cause calculation mistakes.");
-				$CreativeParser->{BeforeOrAfter} = 'before';
-			}
-			if($CreativeParser->{BeforeOrAfter} eq 'before') {
-				# Before parsing
-				# Okay, we need to find the closest $CreativeParser{MustBeDay} before $CreativeParser{FinalYDay}
-				while (1) {
-					if(_Holiday_DayName($CreativeParser->{FinalYDay}, $Year) eq $CreativeParser->{MustBeDay}) {
-						last;
-					}
-					$CreativeParser->{FinalYDay} = $CreativeParser->{FinalYDay} - 1;
-				}
-			} elsif ($CreativeParser->{BeforeOrAfter} eq 'after') {
-				# After parsing
-				# Okay, we need to find the closest $CreativeParser{MustBeDay} after $CreativeParser{FinalYDay}
-				while (1) {
-					if(_Holiday_DayName($CreativeParser->{FinalYDay}, $Year) eq $CreativeParser->{MustBeDay}) {
-						last;
-					}
-					$CreativeParser->{FinalYDay} = $CreativeParser->{FinalYDay} + 1;
-				}
-			} else {
-				_HolidayError($LineNo, $File, "BeforeOrAfter was set to an invalid value ($CreativeParser->{BeforeOrAfter})", "This is a bug in the parser. This line will be ignored.");
-				return(undef);
-			}
-		} else {
-			_SyntaxError($LineNo, $File, "A day is defined but no other way to find out when the day is could be found", "Ignoring this line");
-			return(0);
-		}
-	} 
-	# Calculate the yday of that day-of-the-month
-	elsif(defined($CreativeParser->{IsMonth})) {
-		unless(defined($CreativeParser->{DateNumeric})) {
-				_SyntaxError($LineNo, $File, "It was set which month the day should be on but no information about the day itself ", "Ignoring this line");
-				return(0);
-			}
-		my $PosixYear = $Year - 1900;
-		my $PosixTime = POSIX::mktime(0, 0, 0, $CreativeParser->{DateNumeric}, $MonthMapping{$CreativeParser->{IsMonth}}, $PosixYear);
-		my $proper_yday = _Get_YDay($PosixTime);
-		$CreativeParser->{FinalYDay} = $proper_yday;
-	} 
-	# NumericYDay-only parsing is the simplest solution. This is pure and simple maths
-	elsif(defined($CreativeParser->{NumericYDay})) {
-		# NumericYDay-only parsing is the simplest solution. This is pure and simple maths
-		if(defined($CreativeParser->{MustBeDay})) {
-			_SyntaxError($LineNo, $File, "It was set exactly which day the holiday should occur on and also that it should occur on $CreativeParser->{MustBeDay}", "Ignoring the day requirement");
-
-		}
-		$CreativeParser->{FinalYDay} = _HCalc_NumericYDay($CreativeParser->{NumericYDay}, $CreativeParser->{AddDays}, $CreativeParser->{SubtDays});
-	}
-
-	# Verify the use of the "every" keyword
-	if(defined($CreativeParser->{Every}) and not defined($CreativeParser->{Number})) {
-		_SyntaxError($LineNo, $File, "Use of the \"every\" keyword without any trailing month", "Ignoring the \"every\" keyword.");
-	}
-	if(defined($CreativeParser->{Every}) and defined($CreativeParser->{Length})) {
-		_SyntaxError($LineNo, $File, "Use of both \"every\" and \"length", "This might give unpredictable results.");
-	}
-	# Do the final parsing and add it to the hash
-	if(defined($CreativeParser->{FinalYDay})) {
-		while(1) {
-			if(defined($CreativeParser->{FinalYDay})) {
-				my $PosixYear = $Year - 1900;
-				my ($final_sec,$final_min,$final_hour,$final_mday,$final_mon,$final_year,$final_wday,$final_yday,$final_isdst) = localtime(POSIX::mktime(0, 0, 0, $CreativeParser->{FinalYDay}, 0, $PosixYear));
-				$final_mon++;
-				$FinalParsing->{$final_mon}{$final_mday}{$HolidayName} = $CreativeParser->{HolidayType};
-			} else {
-				last;
-			}
-			if(defined($CreativeParser->{Every}) and defined($CreativeParser->{Number})) {
-				delete($CreativeParser->{Every});
-				if($CreativeParser->{Number} ne "second") {
-					_SyntaxError($LineNo, $File, "Nonsense use of $CreativeParser->{Number} along with \"every\"","Ignoring the \"every\" keyword.");
-				} else {
-					# Add 14 days 
-					$CreativeParser->{FinalYDay} += 14;
-				}
-			}elsif(defined($CreativeParser->{Length}) and $CreativeParser->{Length} > 0) {
-				$CreativeParser->{Length}-- or die("FATAL: attempted to reduce (--) length but it failed! This is a bug.");
-				$CreativeParser->{FinalYDay}++;
-			} else {
-				last;
-			}
-		}
-	} else {
-		_HolidayError($LineNo, $File, "No FinalYDay after finished parsing", "This is a bug in the parser!");
-	}
-	return(1);
-}
-
-# Purpose: Load and parse the holiday file
+# Purpose: Emulate the legacy syntax using the new OO-syntax
 # Usage: Parse(FILE, YEAR);
-sub Parse($$) {
-	my $File = $_[0];
-	my $Year = $_[1];
-
+sub Parse {
+	my ($File, $Year) = @_;
 	carp("$File does not exist") and return(undef) unless -e $File;
 	carp("$File is not readable") and return(undef) unless -r $File;
 	carp("$Year is not numeric") and return(undef) if $Year =~ /\D/;
 
-	my %FinalParsing;
-
 	# Validate year
 	if($Year > 2037) {
 		carp("The holiday parser can't count longer than 2037 due to imitations with the 32bit posix time functions. Ignoring request for paring of $File for the year $Year");
 		return(undef);
 	}
+	my $Object = Date::HolidayParser->new($File);
+	return($Object->parse($Year));
+}
+
+# Purpose: Interperate and calculate the holiday file
+# Usage: $self->_interperate_year(YEAR);
+sub _interperate_year {
+	my $self = $_[0];
+	my $Year = $_[1];
+	my $PosixYear = $Year - 1900;
+	my $FinalParsing = {};
+	foreach my $LineNo (keys(%{$self->{parsed}})) {
+		my ($FinalYDay, $NumericYDay);
+		my $CreativeParser = $self->{parsed}->{$LineNo};
+		my $HolidayName = $self->{parsed}->{$LineNo}{name};
+		my $File = $self->{FILE};
+		my %MonthMapping = (
+			'january' => 0,
+			'february' => 1,
+			'march' => 2,
+			'april' => 3,
+			'may' => 4,
+			'june' => 5,
+			'july' => 6,
+			'august' => 7,
+			'september' => 8,
+			'october' => 9,
+			'november' => 10,
+			'december' => 11,
+		);	# Hash mapping the month name to a numeric
 	
-	my $PosixYear = $Year;
-	$PosixYear -= 1900;
-	open(HOLIDAYFILE, "<$File") or croak("Unable to open $File for reading");
+		if(defined($CreativeParser->{AddEaster})) {
+			$NumericYDay = EasterCalc($Year);
+		}
+		if(defined($CreativeParser->{MonthDay})) {
+			my $month = $CreativeParser->{MonthDay};
+			my $day = $CreativeParser->{MonthDay};
+			$month =~ s/^(\d+)-(\d+)$/$1/;
+			$day =~ s/^(\d+)-(\d+)$/$2/;
+			my $PosixTime = POSIX::mktime(0, 0, 0, $day, $month, $PosixYear);
+			$NumericYDay = _Get_YDay($PosixTime);
+		}
+		
+		unless(defined($CreativeParser->{IsMonth}) or defined($NumericYDay)) {
+			_SyntaxError($LineNo, $File, "I had no day-of-the-year nor a month defined after parsing", "Ignoring this line");
+			next;
+		}
+
+		if(defined($CreativeParser->{MustBeDay})) {
+	
+			# If IsMonth is defined then find a NumericYDay that we can use so that
+			# the NumericYDay parsing below can do all of the heavy lifting
+			if (defined($CreativeParser->{IsMonth}) and defined($CreativeParser->{Number})) {
+				my $PosixYear = $Year - 1900;
+				my $PosixTime = POSIX::mktime(0, 0, 0, 1, $MonthMapping{$CreativeParser->{IsMonth}}, $PosixYear);
+				my $proper_yday = _Get_YDay($PosixTime);
+				unless(defined($CreativeParser->{Number})) {
+					_HolidayError($LineNo, $File, "\$CreativeParser->{Number} is undef", "Skipping this line. This is probably a bug in the parser");
+					next;
+				}
+				if($CreativeParser->{Number} eq 'last') {
+					# Find the first of the set day
+					while(1) {
+						if(_Holiday_DayName($proper_yday, $Year) eq $CreativeParser->{MustBeDay}) {
+							last;
+						}
+						$proper_yday++;
+					}
+	
+					# Find the last of the set day
+					my $Last_YDay = $proper_yday;
+					while(1) {
+						if(_Holiday_DayName($proper_yday, $Year) eq $CreativeParser->{MustBeDay}) {
+							$proper_yday += 7;
+						}
+						my $MKTime = POSIX::mktime(0, 0, 0, $proper_yday, 0, $PosixYear);
+						my ($detect_sec,$detect_min,$detect_hour,$detect_mday,$detect_mon,$detect_year,$detect_wday,$detect_yday,$detect_isdst) = localtime($MKTime);
+						# If $detect_mon is not equal to $MonthMapping{$CreativeParser->{IsMonth}} then
+						# we're now on the next month and have found the last of the day
+						unless($detect_mon eq $MonthMapping{$CreativeParser->{IsMonth}}) {
+							last;
+						}
+						$Last_YDay = $proper_yday;
+					}
+					$NumericYDay = $Last_YDay;
+					$CreativeParser->{BeforeOrAfter} = 'before';
+				} else {
+					# Parse the final
+					$NumericYDay = $proper_yday;
+					if($CreativeParser->{Number} eq 'first') {
+						$CreativeParser->{BeforeOrAfter} = 'after';
+					} elsif($CreativeParser->{Number} eq 'second') {
+						$CreativeParser->{BeforeOrAfter} = 'after';
+						$CreativeParser->{AddDays} = 7;
+					} elsif($CreativeParser->{Number} eq 'third') {
+						$CreativeParser->{BeforeOrAfter} = 'after';
+						$CreativeParser->{AddDays} = 14;
+					} elsif($CreativeParser->{Number} eq 'fourth') {
+						$CreativeParser->{BeforeOrAfter} = 'after';
+						$CreativeParser->{AddDays} = 21;
+					} else {
+						_HolidayError($LineNo, $File, "\$CreativeParer->{Number} is \"$CreativeParser->{Number}\"", "This is a bug in the parser. This line will be ignored") and next unless $CreativeParser->{Number} eq 'null';
+					}
+				}
+			} elsif (defined($CreativeParser->{IsMonth}) and defined($CreativeParser->{DateNumeric})) {
+				my $PosixYear = $Year - 1900;
+				my $PosixTime = POSIX::mktime(0, 0, 0, $CreativeParser->{DateNumeric}, $MonthMapping{$CreativeParser->{IsMonth}}, $PosixYear);
+				$NumericYDay = _Get_YDay($PosixTime);
+			} elsif (defined($CreativeParser->{IsMonth})) {
+				_SyntaxError($LineNo, $File, "There is a month defined but no way to find out which day of the month it is referring to", "Ignoring this line.");
+				next;
+			}
+	
+	
+	
+			if(defined($NumericYDay)) {
+				# Parse the main NumericYDay
+				$FinalYDay = _HCalc_NumericYDay($NumericYDay, $CreativeParser->{AddDays}, $CreativeParser->{SubtDays});
+				unless(defined($CreativeParser->{BeforeOrAfter})) {
+					_SyntaxError($LineNo, $File, "It was not defined if the day should be before or after", "Defaulting to before. This is likely to cause calculation mistakes.");
+					$CreativeParser->{BeforeOrAfter} = 'before';
+				}
+				if($CreativeParser->{BeforeOrAfter} eq 'before') {
+					# Before parsing
+					# Okay, we need to find the closest $CreativeParser{MustBeDay} before $CreativeParser{FinalYDay}
+					while (1) {
+						if(_Holiday_DayName($FinalYDay, $Year) eq $CreativeParser->{MustBeDay}) {
+							last;
+						}
+						$FinalYDay = $FinalYDay - 1;
+					}
+				} elsif ($CreativeParser->{BeforeOrAfter} eq 'after') {
+					# After parsing
+					# Okay, we need to find the closest $CreativeParser{MustBeDay} after $CreativeParser{FinalYDay}
+					while (1) {
+						if(_Holiday_DayName($FinalYDay, $Year) eq $CreativeParser->{MustBeDay}) {
+							last;
+						}
+						$FinalYDay = $FinalYDay + 1;
+					}
+				} else {
+					_HolidayError($LineNo, $File, "BeforeOrAfter was set to an invalid value ($CreativeParser->{BeforeOrAfter})", "This is a bug in the parser. This line will be ignored.");
+					return(undef);
+				}
+			} else {
+				_SyntaxError($LineNo, $File, "A day is defined but no other way to find out when the day is could be found", "Ignoring this line");
+				next;
+			}
+		} 
+		# Calculate the yday of that day-of-the-month
+		elsif(defined($CreativeParser->{IsMonth})) {
+			unless(defined($CreativeParser->{DateNumeric})) {
+					_SyntaxError($LineNo, $File, "It was set which month the day should be on but no information about the day itself ", "Ignoring this line");
+					next;
+			}
+			my $PosixYear = $Year - 1900;
+			my $PosixTime = POSIX::mktime(0, 0, 0, $CreativeParser->{DateNumeric}, $MonthMapping{$CreativeParser->{IsMonth}}, $PosixYear);
+			my $proper_yday = _Get_YDay($PosixTime);
+			$FinalYDay = _HCalc_NumericYDay($proper_yday, $CreativeParser->{AddDays}, $CreativeParser->{SubtDays});
+		}	 
+		# NumericYDay-only parsing is the simplest solution. This is pure and simple maths
+		elsif(defined($NumericYDay)) {
+			# NumericYDay-only parsing is the simplest solution. This is pure and simple maths
+			if(defined($CreativeParser->{MustBeDay})) {
+				_SyntaxError($LineNo, $File, "It was set exactly which day the holiday should occur on and also that it should occur on $CreativeParser->{MustBeDay}", "Ignoring the day requirement");
+	
+			}
+			$FinalYDay = _HCalc_NumericYDay($NumericYDay, $CreativeParser->{AddDays}, $CreativeParser->{SubtDays});
+		}
+	
+		# Verify the use of the "every" keyword
+		if(defined($CreativeParser->{Every}) and not defined($CreativeParser->{Number})) {
+			_SyntaxError($LineNo, $File, "Use of the \"every\" keyword without any trailing month", "Ignoring the \"every\" keyword.");
+		}
+		if(defined($CreativeParser->{Every}) and defined($CreativeParser->{Length})) {
+			_SyntaxError($LineNo, $File, "Use of both \"every\" and \"length", "This might give unpredictable results.");
+		}
+		# Do the final parsing and add it to the hash
+		if(defined($FinalYDay) and $FinalYDay =~ /^\d+$/) {
+			while(1) {
+				if(defined($FinalYDay)) {
+					my $PosixYear = $Year - 1900;
+					my ($final_sec,$final_min,$final_hour,$final_mday,$final_mon,$final_year,$final_wday,$final_yday,$final_isdst) = localtime(POSIX::mktime(0, 0, 0, $FinalYDay, 0, $PosixYear));
+					$final_mon++;
+					$FinalParsing->{$final_mon}{$final_mday}{$HolidayName} = $CreativeParser->{HolidayType};
+				} 
+				if(defined($CreativeParser->{Every}) and defined($CreativeParser->{Number})) {
+					delete($CreativeParser->{Every});
+					if($CreativeParser->{Number} ne "second") {
+						_SyntaxError($LineNo, $File, "Nonsense use of $CreativeParser->{Number} along with \"every\"","Ignoring the \"every\" keyword.");
+					} else {
+						# Add 14 days 
+						$FinalYDay += 14;
+					}
+				}elsif(defined($CreativeParser->{Length}) and $CreativeParser->{Length} > 0) {
+					$CreativeParser->{Length}-- or die("FATAL: attempted to reduce (--) length but it failed! This is a bug.");
+					$FinalYDay++;
+				} else {
+					last;
+				}
+			}
+		} elsif(defined($FinalYDay)) {
+			_HolidayError($LineNo, $File, "Invalid FinalYDay ($FinalYDay) after finished parsing", "This is a bug in the parser!");
+		} else {
+			_HolidayError($LineNo, $File, "No FinalYDay after finished parsing", "This is a bug in the parser!");
+		}
+	}
+	return($FinalParsing);
+}
+
+# Purpose: Load and parse the holiday file
+# Usage: $self->_load_and_parse(FILE);
+sub _load_and_parse {
+	my $self = $_[0];
+	my $File = $_[1];
+
+	carp("$File does not exist") and return(undef) unless -e $File;
+	carp("$File is not readable") and return(undef) unless -r $File;
+
+	my %FinalParsing;
+	open(my $HolidayFile, "<" ,"$File") or croak("Unable to open $File for reading");
 	my $LineNo;
-	foreach my $Line (<HOLIDAYFILE>) {
+	while(my $Line = <$HolidayFile>) {
 		$LineNo++;
 		next if $Line =~ /^\s*[:;#]/;# Ignore these lines
 		next if $Line =~ /^\s*$/;# Ignore lines with only whitespace
@@ -452,9 +485,7 @@ sub Parse($$) {
 					$day =~ s#^\d+/(\d+)\.?$#$1#;
 				}
 				$month--;	# The month in the holiday file is 1-12, we use 0-11
-				my $PosixTime = POSIX::mktime(0, 0, 0, $day, $month, $PosixYear);
-				my ($new_sec,$new_min,$new_hour,$new_mday,$new_mon,$new_year,$new_wday,$new_yday,$new_isdst) = localtime($PosixTime);
-				$CreativeParser{NumericYDay} = $new_yday;
+				$CreativeParser{MonthDay} = "$month-$day";
 			} elsif (/^(january|february|march|april|may|june|july|august|september|october|november|december)$/) {	# Which month it occurs in
 				$CreativeParser{IsMonth} = $_;
 			} elsif (/^plus$/) {			# If the next number should be added to a NumericYDay value
@@ -464,10 +495,11 @@ sub Parse($$) {
 			} elsif (/^length$/) {			# How long (in days) it lasts. FIXME: is currently ignored
 				$CreativeParser{NextIs} = 'length';
 			} elsif (/^easter$/) {			# The day of easter
-				$CreativeParser{NumericYDay} = EasterCalc($Year);
+				$CreativeParser{AddEaster} = 1;
 			} elsif (/^weekend$/) {			# Malplaced weekend keyword
 				$HolidayType = 'red';
 			} elsif (/^\d+$/) {			# Any other number, see below for parsing
+				_SyntaxError($LineNo, $File, "Unreasonably high number", "Ignoring the number. This might give weird results") and next if $_ > 365;
 				# If NextIs is not defined then it's a DateNumeric
 				unless(defined($CreativeParser{NextIs}) and $CreativeParser{NextIs}) {
 					$CreativeParser{DateNumeric} = $_;
@@ -495,11 +527,7 @@ sub Parse($$) {
 					} else {
 						$CreativeParser{Length} = $_;
 					}
-				} else {
-					# What on earth am I going to do with this number?
-					_SyntaxError($LineNo, $File, "Got a number ($_)", "I don't know what to do with this number. Ignoring it.");
 				}
-				$CreativeParser{NextIs} = undef;
 				
 			} elsif (/^(before|after)$/) {	# If a day should be before or after a certain day/date
 				$CreativeParser{BeforeOrAfter} = $_;
@@ -512,32 +540,60 @@ sub Parse($$) {
 			}
 		}
 		
-		unless(defined($CreativeParser{IsMonth}) or defined($CreativeParser{NumericYDay})) {
-			_SyntaxError($LineNo, $File, "I had no day-of-the-year nor a month defined after parsing", "Ignoring this line");
-			next;
-		}
+		# TODO: This isn't an error with the OO interface, but rather the desired behaviour
+
+
+		# ==================================================================
+		# Finalize the interpretation and add it to $self
+		# ==================================================================
 
 		$CreativeParser{HolidayType} = $HolidayType;
-
-		# ==================================================================
-		# Interperate the line
-		# ==================================================================
-
-		_Holiday_Interperate(\%CreativeParser, $LineNo, $HolidayName, $Year, \%FinalParsing, $File);
+		$CreativeParser{name} = $HolidayName;
+		$self->{parsed}{$LineNo} = \%CreativeParser;
 	}
-	return(\%FinalParsing);
+	close($HolidayFile);
 }
 
+# Purpose: Create a new object and call _load_and_parse on it
+# Usage: my $object = Date::HolidayParser->new(/FILE/);
+sub new {
+	unless(defined($_[1])) {
+		carp("Needs an option: path to the holiday file");
+		return(undef);
+	}
+	unless(-e $_[1]) {
+		carp("\"$_[1]\": does not exist");
+		return(undef);
+	}
+	my $self = {};
+	bless($self);
+	$self->{FILE} = $_[1];
+	$self->_load_and_parse($self->{FILE});
+	return($self);
+}
+
+# Purpose: Get the holiday information for YEAR
+# Usage: my $HashRef = $object->get(YEAR);
+sub get {
+	my $self = $_[0];
+	carp("Date::HolidayParser->get needs an parameter: The year to parse") and return(undef) unless(defined($_[1]));
+	my $Year = $_[1];
+	carp("Date::HolidayParser: The year must be a digit") and return(undef) if $Year =~ /\D/;
+	carp("Date::HolidayParser: Can't parse years lower than 1971") and return(undef) if $Year < 1971;
+	carp("Date::HolidayParser: Can't parse years higher than 2037") and return(undef) if $Year > 2037;
+	return($self->_interperate_year($Year));
+}
 # End of Date::HolidayParser
 1;
 
+__END__
 =head1 NAME
 
 Date::HolidayParser - Parser for .holiday-files
 
 =head1 VERSION
 
-0.1
+0.3
 
 =head1 SYNOPSIS
 
@@ -549,7 +605,8 @@ This module returns a hash that you can read and use within your program.
 
 	use Date::HolidayParser;
 
-	my $Holidays = Date::HolidayParser::Parse("$ENV{HOME}/.holiday", 2006);
+	my $Holidays = Date::HolidayParser->new("$ENV{HOME}/.holiday");
+	my $Holidays_2006 = $Holidays->get(2006);
 	
 	...
 
@@ -565,27 +622,29 @@ within your perl program in whatever way you wish.
 
 =head1 EXPORT
 
-This module doesn't export anything by default. You're encouraged to directly use the
-module functions by issuing Date::HolidayParser::Function. It can however export the
-Parse and EasterCalc functions upon request by issuing
+This module doesn't export anything by default. You're encouraged to use the object
+oriented interface. It can however export the Parse and EasterCalc functions upon
+request by issuing
 
 	use Date::HolidayParser qw(EasterCalc Parse);
 	...
 
 =head1 FUNCTIONS
 
-=head2 Date::HolidayParser::Parse
+=head2 $object = Date::HolidayParser->new(FILE);
 
-This is the primary function of Date::HolidayParser. Its syntax is:
+This is the main function. It creates a new Date::HolidayParser object for FILE and
+parses the file.
 
-	use Date::HolidayParser;
+FILE must be the full path to the holiday file you want to parse.
 
-	my $Holidays = Date::HolidayParser::Parse("/path/to/holiday.file", "YEAR");
+=head2 $object->get(YEAR);
+
+This gets the holidays for YEAR. It uses the already parsed FILE and calculates the
+holidays in YEAR and returns a hashref with the parsed data or undef on failure.
 
 YEAR must be a full year (ie. 2006) not a year relative to 1900 (ie. 106).
-The path must be the full path to the holiday file you want to parse.
 
-It returns a hashref with the parsed data or undef on failure.
 See the section HASH SYNTAX below for the syntax of the returned hashref.
 
 =head2 Date::HolidayParser::EasterCalc
@@ -604,6 +663,22 @@ It returns the day of easter of the year supplied.
 NOTE: The day returned begins on 0. This means that the days returns
 are 0-364 instead of 1-365.
 
+=head2 Date::HolidayParser::Parse
+
+This is the legacy syntax of Date::HolidayParser. You're strongly encouraged to use
+the object oriented interface. This function is slower than the OO-interface if you
+need to interperate more than one year.
+
+	use Date::HolidayParser;
+
+	my $Holidays = Date::HolidayParser::Parse("/path/to/holiday.file", "YEAR");
+
+YEAR must be a full year (ie. 2006) not a year relative to 1900 (ie. 106).
+The path must be the full path to the holiday file you want to parse.
+
+It returns a hashref with the parsed data or undef on failure.
+See the section HASH SYNTAX below for the syntax of the returned hashref.
+
 =head1 HASH SYNTAX
 
 The returned hash is in the following format:
@@ -668,15 +743,16 @@ The UK holiday file was chosen because it is rather small and simple.
 	use Date::HolidayParser;
 	
 	# Call Date::HolidayParser to parse the file
-	my $Holidays = Date::HolidayParser::Parse("$ENV{HOME}/holidays/holiday_uk",2006);
+	my $Holidays = Date::HolidayParser->new();
+	my $Holidays_2006 = $Holidays->get(2006);
 
 	# Set a proper Data::Dumper format and dump the data returned by Date::HolidayParser to STDOUT
 	$Data::Dumper::Purity = 1; $Data::Dumper::Sortkeys = 1; $Data::Dumper::Indent = 1;
-	print Data::Dumper->Dump([$Holidays], ["*Holidays"]);
+	print Data::Dumper->Dump([$Holidays_2006], ["*Holidays_2006"]);
 
 =head2 The output
 
-	%Holidays = (
+	%Holidays_2006 = (
 	  '1' => {
 	    '1' => {
 	      'New Years Day' => 'red'
@@ -719,9 +795,10 @@ The UK holiday file was chosen because it is rather small and simple.
 
 =head2 Explenation
 
-This is a very simple example. It first uses Date::HolidayParser to parse the file
-and then save the hashref returned to $Holiday. Then it tells Data::Dumper to dump
-a visual (perl-usable) representtion of the hash to stdout.
+This is a very simple example. It first creates a $Holidays Date::HolidayParser object,
+then tells it to get the holidays for the year 2006 ($Holidays->get(2006);) and saves
+the information to $Holidays_2006. Then it tells Data::Dumper to dump
+a visual (perl-usable) representation of the hash to stdout.
 
 =head1 SETTINGS
 
@@ -743,7 +820,7 @@ L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Date-HolidayParser>.
 I will be notified, and then you'll automatically be notified of progress on
 your bug as I make changes.
 
-=head1 COPYRIGHT & LICENSE
+=head1 LICENSE AND COPYRIGHT
 
 Copyright (C) 2006 Eskild Hustvedt, all rights reserved.
 
